const CANVAS_WIDTH = 800;
const CANVAS_HEIGHT = 600;
const CANVAS_SELECTOR = "[data-test=canvas]";

describe("Viewer2d", () => {
  beforeEach(() => {
    cy.visit("test");
    cy.window().its("viewer2d").as("viewer2d");
    cy.get("@viewer2d").then(viewer2d => {
      const model = {
        bbox: [0, 0, 150, 150],
        objects: [
          {
            type: "testType1",
            area: 1,
            path: new Path2D("M0 0 v 50 h 50 v -50 Z"),
          },
          {
            type: "testType2",
            area: 2,
            path: new Path2D(
              "M 75 75 m -75, 0 a 75,75 0 1,0 150,0 a 75,75 0 1,0 -150,0"
            ),
          },
        ],
      };
      viewer2d.loadModel(model);

      viewer2d.camera.fitView(model.bbox);

      viewer2d.on("pick", ({ object, rightClick }) => {
        if (rightClick) {
          viewer2d.camera.fitView(object.bbox);
        } else {
          viewer2d.scene.setObjectsSelected([object.id], !object.selected);
        }
      });
      viewer2d.on("hover", ({ object }) =>
        viewer2d.scene.setObjectsHighlighted([object.id], true)
      );
      viewer2d.on("hover-out", ({ object }) =>
        viewer2d.scene.setObjectsHighlighted([object.id], false)
      );
    });
  });

  it("Should highlight object on hover", () => {
    cy.get(CANVAS_SELECTOR).trigger("mousemove", {
      offsetX: 95,
      offsetY: CANVAS_HEIGHT / 2,
    });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.highlighted).length
      ).to.equal(0);
    });
    cy.get(CANVAS_SELECTOR).trigger("mousemove", {
      offsetX: 105,
      offsetY: CANVAS_HEIGHT / 2,
    });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.highlighted).length
      ).to.equal(1);
    });
    cy.get(CANVAS_SELECTOR).trigger("mousemove", {
      offsetX: 695,
      offsetY: CANVAS_HEIGHT / 2,
    });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.highlighted).length
      ).to.equal(1);
    });
    cy.get(CANVAS_SELECTOR).trigger("mousemove", {
      offsetX: 705,
      offsetY: CANVAS_HEIGHT / 2,
    });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.highlighted).length
      ).to.equal(0);
    });
  });

  it("Should select/deselect object on click", () => {
    // Click on nothing
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 95, offsetY: CANVAS_HEIGHT / 2 })
      .trigger("mouseup", { offsetX: 95, offsetY: CANVAS_HEIGHT / 2 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(0);
    });

    // Click on object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 105, offsetY: CANVAS_HEIGHT / 2 })
      .trigger("mouseup", { offsetX: 105, offsetY: CANVAS_HEIGHT / 2 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(1);
    });

    // Click on the same object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 695, offsetY: CANVAS_HEIGHT / 2 })
      .trigger("mouseup", { offsetX: 695, offsetY: CANVAS_HEIGHT / 2 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(0);
    });

    // Click on nothing
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 705, offsetY: CANVAS_HEIGHT / 2 })
      .trigger("mouseup", { offsetX: 705, offsetY: CANVAS_HEIGHT / 2 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(0);
    });

    // Click on the first object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", {
        offsetX: CANVAS_WIDTH / 2,
        offsetY: CANVAS_HEIGHT / 2,
      })
      .trigger("mouseup", {
        offsetX: CANVAS_WIDTH / 2,
        offsetY: CANVAS_HEIGHT / 2,
      });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(1);
    });

    // Click on the second object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 105, offsetY: 0 })
      .trigger("mouseup", { offsetX: 105, offsetY: 0 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(2);
    });

    // Click on the first object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", {
        offsetX: CANVAS_WIDTH / 2,
        offsetY: CANVAS_HEIGHT / 2,
      })
      .trigger("mouseup", {
        offsetX: CANVAS_WIDTH / 2,
        offsetY: CANVAS_HEIGHT / 2,
      });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(1);
    });

    // Click on the second object
    cy.get(CANVAS_SELECTOR)
      .trigger("mousedown", { offsetX: 105, offsetY: 0 })
      .trigger("mouseup", { offsetX: 105, offsetY: 0 });
    cy.get("@viewer2d").should(viewer2d => {
      expect(
        viewer2d.scene.objects.filter(object => object.selected).length
      ).to.equal(0);
    });
  });
});
