/**
 * A factory to create a viewer2d
 */
declare module "@bimdata/viewer2d" {
  export default Viewer2DFactory;

  function Viewer2DFactory(cfg: Viewer2DCfg): Viewer2D;

  interface Viewer2DCfg {
    canvas: HTMLCanvasElement;
    rotateKey?: string;
    scaleSpeed?: number;
    rotateSpeed?: number;
    maxScale?: number;
    minScale?: number;
    fitViewRatio?: number;
    arrowMove?: number;
  }

  interface Viewer2D {
    readonly materials: object;
    loadModel(model: Model): Model;
    unloadModel(id: number): boolean;
    scene: Scene;
    camera: Camera;
    destroy(): void;
    on(event: string, callback: function): number;
    once(event: string, callback: function): number;
    off(subscriptionId: number): void;
  }

  interface Scene {
    readonly models: Model[];
    readonly objects: Object[];
    setObjectsVisible(ids: number[], test: boolean): void;
    setObjectsSelected(ids: number[], selected: boolean): void;
    setObjectsHighlighted(ids: number[], highlighted: boolean): void;
    setObjectsPickable(ids: number[], pickable: boolean): void;
  }

  interface Camera {
    fitView(bbox: number[]): void;
  }

  interface Model {
    objects: Object[];
  }

  interface Object {
    path: Path2D;
    area?: number;
    type?: string;
  }
}
